@@ -21,27 +21,6 @@ import PyKCS11.LowLevel
 import os
 import sys
 
-PY3 = sys.version_info[0] >= 3
-if PY3:
-    def byte_to_int(byte):
-        return byte
-
-    def to_param_string(param):
-        if isinstance(param, str):
-            return bytes(param, 'ascii')
-        else:
-            return bytes(param)
-else:
-    def byte_to_int(byte):
-        return ord(byte)
-
-    def to_param_string(param):
-        if isinstance(param, str):
-            return param
-        else:
-            return str(bytearray(param))
-
-    range = xrange
 
 # redefine PKCS#11 constants
 CK_TRUE = PyKCS11.LowLevel.CK_TRUE
@@ -82,6 +61,7 @@ for x in PyKCS11.LowLevel.__dict__.keys():
             eval(x[:3])[x] = eval(x)  # => CKM['CKM_RSA_PKCS'] = CKM_RSA_PKCS
 
 # special CKR[] values
+CKR[-3] = "Unknown format"
 CKR[-2] = "Unkown PKCS#11 type"
 CKR[-1] = "Load"
 
@@ -91,6 +71,46 @@ class ckbytelist(PyKCS11.LowLevel.ckbytelist):
     add a __repr__() method to the LowLevel equivalent
     """
 
+    def __init__(self, data=[]):
+        # default size of the vector
+        size = 0
+        if isinstance(data, int):
+            size = data
+            data = None
+
+        super(ckbytelist, self).__init__(size)
+
+        # No value to initialize
+        if data is None:
+            return
+
+        # b'abc'
+        if isinstance(data, bytes):
+            self.reserve(len(data))
+            for x in data:
+                if sys.version_info[0] <= 2:
+                    # Python 2
+                    v = ord(x)
+                else:
+                    # Python 3 and more
+                    v = x
+                self.append(v)
+
+        # "abc"
+        elif isinstance(data, str):
+            tmp = bytes(data, "utf-8")
+            self.reserve(len(tmp))
+            for x in tmp:
+                self.append(x)
+
+        # [141, 142, 143]
+        elif isinstance(data, list) or isinstance(data, ckbytelist):
+            self.reserve(len(data))
+            for c in range(len(data)):
+                self.append(data[c])
+        else:
+            raise PyKCS11.PyKCS11Error(-3, text=type(data))
+
     def __repr__(self):
         """
         return the representation of a tuple
@@ -434,7 +454,11 @@ class PyKCS11Lib(object):
         self.lib = PyKCS11.LowLevel.CPKCS11Lib()
 
     def __del__(self):
-        self.lib.Unload()
+        if PyKCS11 and PyKCS11.__name__ and \
+                PyKCS11.LowLevel and PyKCS11.LowLevel.__name__ and \
+                PyKCS11.LowLevel._LowLevel and \
+                PyKCS11.LowLevel._LowLevel.__name__:
+            self.lib.Unload()
 
     def load(self, pkcs11dll_filename=None, *init_string):
         """
@@ -450,7 +474,7 @@ class PyKCS11Lib(object):
             pkcs11dll_filename = os.getenv("PYKCS11LIB")
             if pkcs11dll_filename is None:
                 raise PyKCS11Error(-1, "No PKCS11 library specified (set PYKCS11LIB env variable)")
-        rv = self.lib.Load(pkcs11dll_filename, True)
+        rv = self.lib.Load(pkcs11dll_filename)
         if rv == 0:
             raise PyKCS11Error(-1, pkcs11dll_filename)
 
@@ -486,15 +510,19 @@ class PyKCS11Lib(object):
         i.libraryVersion = (info.libraryVersion.major, info.libraryVersion.minor)
         return i
 
-    def getSlotList(self):
+    def getSlotList(self, tokenPresent=False):
         """
         C_GetSlotList
 
+        @param tokenPresent: L{False} (default) to list all slots,
+        L{True} to list only slots with present tokens
+        @type tokenPresent: bool
         @return: a list of available slots
         @rtype: list
         """
         slotList = PyKCS11.LowLevel.ckintlist()
-        rv = self.lib.C_GetSlotList(0, slotList)
+        rv = self.lib.C_GetSlotList(CK_TRUE if tokenPresent else CK_FALSE,
+                slotList)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
 
@@ -583,7 +611,7 @@ class PyKCS11Lib(object):
 
         @param slot: slot number returned by L{getSlotList}
         @type slot: integer
-        @param flags: 0 (default), L{CKF_RW_SESSION} for RW session
+        @param flags: 0 (default), CKF_RW_SESSION for RW session
         @type flags: integer
         @return: a L{Session} object
         """
@@ -685,10 +713,10 @@ class Mechanism(object):
         """
         self._mech = PyKCS11.LowLevel.CK_MECHANISM()
         self._mech.mechanism = mechanism
-        self._param = None 
+        self._param = None
         if param:
-            self._param = to_param_string(param)
-            self._mech.pParameter = self._param 
+            self._param = ckbytelist(param)
+            self._mech.pParameter = self._param
             self._mech.ulParameterLen = len(param)
 
     def to_native(self):
@@ -700,24 +728,28 @@ MechanismRSAGENERATEKEYPAIR = Mechanism(CKM_RSA_PKCS_KEY_PAIR_GEN, None)
 MechanismECGENERATEKEYPAIR = Mechanism(CKM_EC_KEY_PAIR_GEN, None)
 MechanismAESGENERATEKEY = Mechanism(CKM_AES_KEY_GEN, None)
 
+
 class RSAOAEPMechanism(object):
     """RSA OAEP Wrapping mechanism"""
 
-    def __init__(self, hash, mgf, label=None):
+    def __init__(self, hashAlg, mgf, label=None):
         """
-        @param hash: the hash algorithm to use
-        @param mfg: the mask generation function to use
+        @param hashAlg: the hash algorithm to use (like CKM_SHA256)
+        @param mgf: the mask generation function to use (like
+        CKG_MGF1_SHA256)
         @param label: the (optional) label to use
         """
         self._param = PyKCS11.LowLevel.CK_RSA_PKCS_OAEP_PARAMS()
-        self._param.hashAlg = hash
+        self._param.hashAlg = hashAlg
         self._param.mgf = mgf
         self._source = None
+        self._param.src = CKZ_DATA_SPECIFIED
         if label:
-            self._param.src = CKZ_DATA_SPECIFIED
-            self._source = to_param_string(label)
-            self._param.pSourceData = self._source
-            self._param.ulSourceDataLen = len(label)
+            self._source = ckbytelist(label)
+            self._param.ulSourceDataLen = len(self._source)
+        else:
+            self._param.ulSourceDataLen = 0
+        self._param.pSourceData = self._source
         self._mech = PyKCS11.LowLevel.CK_MECHANISM()
         self._mech.mechanism = CKM_RSA_PKCS_OAEP
         self._mech.pParameter = self._param
@@ -726,6 +758,31 @@ class RSAOAEPMechanism(object):
     def to_native(self):
         return self._mech
 
+
+class RSA_PSS_Mechanism(object):
+    """RSA PSS Wrapping mechanism"""
+
+    def __init__(self, hashAlg, mgf, sLen):
+        """
+        @param hashAlg: the hash algorithm to use (like CKM_SHA384)
+        @param mgf: the mask generation function to use (like
+        CKG_MGF1_SHA384)
+        @param sLen: length, in bytes, of the salt value used in the PSS
+        encoding
+        """
+        self._param = PyKCS11.LowLevel.CK_RSA_PKCS_PSS_PARAMS()
+        self._param.hashAlg = hashAlg
+        self._param.mgf = mgf
+        self._param.sLen = sLen
+        self._mech = PyKCS11.LowLevel.CK_MECHANISM()
+        self._mech.mechanism = CKM_RSA_PKCS_PSS
+        self._mech.pParameter = self._param
+        self._mech.ulParameterLen = PyKCS11.LowLevel.CK_RSA_PKCS_PSS_PARAMS_LENGTH
+
+    def to_native(self):
+        return self._mech
+
+
 class DigestSession(object):
     def __init__(self, lib, session, mecha):
         self._lib = lib
@@ -742,14 +799,7 @@ class DigestSession(object):
         @param data: data to add to the digest
         @type data: bytes or string
         """
-        data1 = ckbytelist()
-        data1.reserve(len(data))
-        if isinstance(data, bytes):
-            for x in data:
-                data1.append(byte_to_int(x))
-        else:
-            for c in range(len(data)):
-                data1.append(data[c])
+        data1 = ckbytelist(data)
         rv = self._lib.C_DigestUpdate(self._session, data1)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
@@ -760,7 +810,7 @@ class DigestSession(object):
         C_DigestKey
 
         @param handle: key handle
-        @type data: Handle
+        @type handle: CK_OBJECT_HANDLE
         """
         rv = self._lib.C_DigestKey(self._session, handle)
         if rv != CKR_OK:
@@ -785,6 +835,7 @@ class DigestSession(object):
             raise PyKCS11Error(rv)
         return digest
 
+
 class Session(object):
     """ Manage L{PyKCS11Lib.openSession} objects """
 
@@ -934,16 +985,8 @@ class Session(object):
 
         """
         digest = ckbytelist()
-        ps = None  # must be declared here or may be deallocated too early
         m = mecha.to_native()
-        data1 = ckbytelist()
-        data1.reserve(len(data))
-        if isinstance(data, bytes):
-            for x in data:
-                data1.append(byte_to_int(x))
-        else:
-            for c in range(len(data)):
-                data1.append(data[c])
+        data1 = ckbytelist(data)
         rv = self.lib.C_DigestInit(self.session, m)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
@@ -978,14 +1021,7 @@ class Session(object):
         """
         m = mecha.to_native()
         signature = ckbytelist()
-        data1 = ckbytelist()
-        data1.reserve(len(data))
-        if isinstance(data, bytes):
-            for x in data:
-                data1.append(byte_to_int(x))
-        else:
-            for c in range(len(data)):
-                data1.append(data[c])
+        data1 = ckbytelist(data)
         rv = self.lib.C_SignInit(self.session, m, key)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
@@ -1017,15 +1053,7 @@ class Session(object):
 
         """
         m = mecha.to_native()
-        data1 = ckbytelist()
-        data1.reserve(len(data))
-
-        if isinstance(data, bytes):
-            for x in data:
-                data1.append(byte_to_int(x))
-        else:
-            for c in range(len(data)):
-                data1.append(data[c])
+        data1 = ckbytelist(data)
         rv = self.lib.C_VerifyInit(self.session, m, key)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
@@ -1057,16 +1085,8 @@ class Session(object):
 
         """
         encrypted = ckbytelist()
-        ps = None  # must be declared here or may be deallocated too early
         m = mecha.to_native()
-        data1 = ckbytelist()
-        data1.reserve(len(data))
-        if isinstance(data, bytes):
-            for x in data:
-                data1.append(byte_to_int(x))
-        else:
-            for c in range(len(data)):
-                data1.append(data[c])
+        data1 = ckbytelist(data)
         rv = self.lib.C_EncryptInit(self.session, m, key)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
@@ -1101,14 +1121,7 @@ class Session(object):
         """
         m = mecha.to_native()
         decrypted = ckbytelist()
-        data1 = ckbytelist()
-        data1.reserve(len(data))
-        if isinstance(data, bytes):
-            for x in data:
-                data1.append(byte_to_int(x))
-        else:
-            for c in range(len(data)):
-                data1.append(data[c])
+        data1 = ckbytelist(data)
         rv = self.lib.C_DecryptInit(self.session, m, key)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
@@ -1141,7 +1154,6 @@ class Session(object):
             ''.join(chr(i) for i in ckbytelistData)
 
         """
-        m = PyKCS11.LowLevel.CK_MECHANISM()
         wrapped = ckbytelist()
         native = mecha.to_native()
         # first call get wrapped size
@@ -1171,15 +1183,7 @@ class Session(object):
 
         """
         m = mecha.to_native()
-        wrapped = ckbytelist()
-        data1 = ckbytelist()
-        data1.reserve(len(wrappedKey))
-        if isinstance(wrappedKey, bytes):
-            for x in wrappedKey:
-                data1.append(byte_to_int(x))
-        else:
-            for c in range(len(wrappedKey)):
-                data1.append(wrappedKey[c])
+        data1 = ckbytelist(wrappedKey)
         handle = PyKCS11.LowLevel.CK_OBJECT_HANDLE()
         attrs = self._template2ckattrlist(template)
         rv = self.lib.C_UnwrapKey(self.session, m, unwrappingKey, data1, attrs, handle)
@@ -1273,10 +1277,7 @@ class Session(object):
                 if isinstance(attr[1], int):
                     attrStr = str(attr[1])
                 if isinstance(attr[1], bytes):
-                    attrBin = ckbytelist()
-                    attrBin.reserve(len(attrStr))
-                    for c in range(len(attrStr)):
-                        attrBin.append(byte_to_int(attrStr[c]))
+                    attrBin = ckbytelist(attrStr)
                 t[x].SetBin(attr[0], attrBin)
             else:
                 raise PyKCS11Error(-2)
