@@ -707,6 +707,12 @@ struct ck_rsa_pkcs_oaep_params {
   unsigned long source_data_len;
 } ;
 
+typedef struct ck_rsa_pkcs_pss_params {
+  unsigned long hashAlg;
+  unsigned long mgf;
+  unsigned long sLen;
+} ;
+
 #define CKF_HW			(1 << 0)
 #define CKF_ENCRYPT		(1 << 8)
 #define CKF_DECRYPT		(1 << 9)
@@ -1273,6 +1279,9 @@ typedef struct ck_mechanism_info *CK_MECHANISM_INFO_PTR;
 typedef struct ck_rsa_pkcs_oaep_params CK_RSA_PKCS_OAEP_PARAMS;
 typedef struct ck_rsa_pkcs_oaep_params *CK_RSA_PKCS_OAEP_PARAMS_PTR;
 
+typedef struct ck_rsa_pkcs_pss_params CK_RSA_PKCS_PSS_PARAMS;
+typedef struct ck_rsa_pkcs_pss_params *CK_RSA_PKCS_PSS_PARAMS_PTR;
+
 typedef struct ck_function_list CK_FUNCTION_LIST;
 typedef struct ck_function_list *CK_FUNCTION_LIST_PTR;
 typedef struct ck_function_list **CK_FUNCTION_LIST_PTR_PTR;
@@ -1347,6 +1356,8 @@ typedef struct ck_c_initialize_args *CK_C_INITIALIZE_ARGS_PTR;
 
 #undef ck_rsa_pkcs_oaep_params
 
+#undef ck_rsa_pkcs_pss_params
+
 #undef ck_rv_t
 #undef ck_notify_t
 
